From 5b3e224922eca93711adac5d5201c6d7ae7fbe0f Mon Sep 17 00:00:00 2001
From: Fabio Ritrovato <exsephiroth87@gmail.com>
Date: Mon, 9 Nov 2009 13:30:37 +0100
Subject: [PATCH] [GSoC] luasd: the lua SD module

---
 modules/misc/lua/services_discovery.c |  246 +++++++++++++++++++++++++++++++++
 1 files changed, 246 insertions(+), 0 deletions(-)
 create mode 100644 modules/misc/lua/services_discovery.c

diff --git a/modules/misc/lua/services_discovery.c b/modules/misc/lua/services_discovery.c
new file mode 100644
index 0000000..870aa06
--- /dev/null
+++ b/modules/misc/lua/services_discovery.c
@@ -0,0 +1,246 @@
+/*****************************************************************************
+ * services_discovery.c : Services discovery using lua scripts
+ *****************************************************************************
+ * Copyright (C) 2009 the VideoLAN team
+ *
+ * Authors: Fabio Ritrovato <exsephiroth87@gmail.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <vlc_common.h>
+#include <vlc_services_discovery.h>
+#include <vlc_playlist.h>
+
+#include "vlc.h"
+#include "libs.h"
+
+/*****************************************************************************
+ * Local prototypes
+ *****************************************************************************/
+
+static void *Run( void * );
+static int ItemPut( services_discovery_t *, const char * );
+static int ItemGet( services_discovery_t *, const char *, input_item_t * );
+static int ItemDelete( services_discovery_t *, input_item_t * );
+
+/*****************************************************************************
+ * Local structures
+ *****************************************************************************/
+
+struct services_discovery_sys_t
+{
+    lua_State *L;
+    char *psz_filename;
+    vlc_thread_t thread;
+};
+static const luaL_Reg p_reg[] = { { NULL, NULL } };
+
+/*****************************************************************************
+ * Open: initialize and create stuff
+ *****************************************************************************/
+int Open_LuaSD( vlc_object_t *p_this )
+{
+    services_discovery_t *p_sd = ( services_discovery_t * )p_this;
+    services_discovery_sys_t *p_sys;
+    lua_State *L;
+
+    if( !( p_sys = malloc( sizeof( services_discovery_sys_t ) ) ) )
+        return VLC_ENOMEM;
+    p_sd->p_sys = p_sys;
+    if( asprintf( &p_sys->psz_filename,
+                  "%s" DIR_SEP "lua" DIR_SEP "sd" DIR_SEP "%s.lua",
+                   config_GetUserDir( VLC_DATA_DIR ), p_sd->psz_name ) < 0 )
+    {
+        free( p_sys );
+        return VLC_ENOMEM;
+    }
+    L = luaL_newstate();
+    if( !L )
+    {
+        msg_Err( p_sd, "Could not create new Lua State" );
+        free( p_sys->psz_filename );
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    luaL_openlibs( L );
+    luaL_register( L, "vlc", p_reg );
+    lua_pushlightuserdata( L, p_sd );
+    lua_setfield( L, -2, "private" );
+    luaopen_input( L );
+    luaopen_msg( L );
+    luaopen_misc( L );
+    luaopen_net( L );
+    luaopen_object( L );
+    luaopen_playlist( L );
+    luaopen_sd( L );
+    luaopen_strings( L );
+    luaopen_variables( L );
+    luaopen_stream( L );
+    lua_pop( L, 1 );
+    if( luaL_dofile( L, p_sys->psz_filename ) )
+    {
+
+        msg_Err( p_sd, "Error loading script %s: %s", p_sys->psz_filename,
+                  lua_tostring( L, lua_gettop( L ) ) );
+        lua_pop( L, 1 );
+        free( p_sys->psz_filename );
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    p_sys->L = L;
+    if( vlc_clone (&p_sd->p_sys->thread, Run, p_sd, VLC_THREAD_PRIORITY_LOW) )
+    {
+        free( p_sys->psz_filename );
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    return VLC_SUCCESS;
+}
+
+/*****************************************************************************
+ * Close: cleanup
+ *****************************************************************************/
+void Close_LuaSD( vlc_object_t *p_this )
+{
+    services_discovery_t *p_sd = ( services_discovery_t * )p_this;
+
+    vlc_cancel (p_sd->p_sys->thread);
+    vlc_join (p_sd->p_sys->thread, NULL);
+    free( p_sd->p_sys->psz_filename );
+    lua_close( p_sd->p_sys->L );
+    free( p_sd->p_sys );
+}
+
+/*****************************************************************************
+ * Run: Thread entry-point
+ ****************************************************************************/
+static void* Run( void *data )
+{
+    services_discovery_t *p_sd = ( services_discovery_t * )data;
+    services_discovery_sys_t *p_sys = p_sd->p_sys;
+    lua_State *L = p_sys->L;
+
+    lua_getglobal( L, "main" );
+    if( !lua_isfunction( L, lua_gettop( L ) ) || lua_pcall( L, 0, 1, 0 ) )
+    {
+        msg_Err( p_sd, "Error while runing script %s, "
+                  "function main(): %s", p_sys->psz_filename,
+                  lua_tostring( L, lua_gettop( L ) ) );
+        lua_pop( L, 1 );
+        return NULL;
+    }
+    lua_getglobal( L, "item_put" );
+    if( lua_isfunction( L, lua_gettop( L ) ) )
+        p_sd->pf_item_put = ItemPut;
+    lua_getglobal( L, "item_get" );
+    if( lua_isfunction( L, lua_gettop( L ) ) )
+        p_sd->pf_item_get = ItemGet;
+    lua_getglobal( L, "item_delete" );
+    if( lua_isfunction( L, lua_gettop( L ) ) )
+        p_sd->pf_item_delete = ItemDelete;
+    msg_Info( p_sd, "LuaSD script loaded: %s", p_sd->psz_name );
+    return NULL;
+}
+
+/*****************************************************************************
+ * Item oprations callbacks
+ ****************************************************************************/
+static int ItemPut( services_discovery_t *p_sd, const char *psz_string )
+{
+    lua_State *L = p_sd->p_sys->L;
+
+    lua_getglobal( L, "item_put" );
+    if( !lua_isfunction( L, lua_gettop( L ) ) )
+    {
+        msg_Err( p_sd, "Error while runing script %s, "
+                  "function item_put() not found", p_sd->p_sys->psz_filename );
+        lua_pop( L, 1 );
+        return VLC_EGENERIC;
+    }
+    lua_pushstring( L, psz_string );
+    if( lua_pcall( L, 1, 1, 0 ) )
+    {
+        msg_Err( p_sd, "Error while runing script %s, "
+                  "function item_put(): %s", p_sd->p_sys->psz_filename,
+                  lua_tostring( L, lua_gettop( L ) ) );
+        lua_pop( L, 1 );
+        return VLC_EGENERIC;
+    }
+    return VLC_SUCCESS;
+}
+
+static int ItemGet( services_discovery_t *p_sd, const char *psz_string,
+                    input_item_t *p_item )
+{
+    lua_State *L = p_sd->p_sys->L;
+
+    lua_getglobal( L, "item_get" );
+    if( !lua_isfunction( L, lua_gettop( L ) ) )
+    {
+        msg_Err( p_sd, "Error while runing script %s, "
+                  "function item_get() not found",p_sd->p_sys->psz_filename );
+        lua_pop( L, 1 );
+        return VLC_EGENERIC;
+    }
+    /* TODO: is this the best way to do it? */
+    lua_pushstring( L, input_item_GetURL( p_item ) );
+    lua_pushstring( L, psz_string );
+    if( lua_pcall( L, 2, 1, 0 ) )
+    {
+        msg_Err( p_sd, "Error while runing script %s, "
+                  "function item_get(): %s", p_sd->p_sys->psz_filename,
+                  lua_tostring( L, lua_gettop( L ) ) );
+        lua_pop( L, 1 );
+        return VLC_EGENERIC;
+    }
+    return VLC_SUCCESS;
+}
+
+static int ItemDelete( services_discovery_t *p_sd, input_item_t *p_item )
+{
+    lua_State *L = p_sd->p_sys->L;
+
+    lua_getglobal( L, "item_delete" );
+    if( !lua_isfunction( L, lua_gettop( L ) ) )
+    {
+        msg_Err( p_sd, "Error while runing script %s, "
+                  "function item_delete() not found", p_sd->p_sys->psz_filename );
+        lua_pop( L, 1 );
+        return VLC_EGENERIC;
+    }
+    input_item_t **udata = (input_item_t **)
+                           lua_newuserdata( L, sizeof( input_item_t * ) );
+    *udata = p_item;
+    if( luaL_newmetatable( L, "input_item_t" ) )
+    {
+        lua_pushstring( L, "none of your business" );
+        lua_setfield( L, -2, "__metatable" );
+    }
+    lua_setmetatable( L, -2 );
+    if( lua_pcall( L, 1, 1, 0 ) )
+    {
+        msg_Err( p_sd, "Error while runing script %s, "
+                  "function item_delete(): %s", p_sd->p_sys->psz_filename,
+                  lua_tostring( L, lua_gettop( L ) ) );
+        lua_pop( L, 1 );
+        return VLC_EGENERIC;
+    }
+    return VLC_SUCCESS;
+}
-- 
1.6.0.4

